@@ -12,9 +12,12 @@ use MIME::Base64;
 use Number::Format;
 use Time::HiRes 'time';
 use URI::Escape;
-use XML::Simple;
 
-use Zanas::Presentation::Skins::Classic;
+#use Zanas::Presentation::Skins::Classic;
+#use Zanas::Presentation::Skins::Dumper;
+#use Zanas::Presentation::Skins::XMLDumper;
+#use Zanas::Presentation::Skins::XMLProto;
+#use Zanas::Presentation::Skins::XL;
 
 use constant OK => 200;
 
@@ -24,15 +27,14 @@ BEGIN {
 
 	$Data::Dumper::Sortkeys = 1;
 
-	$Zanas_VERSION      = $Zanas::VERSION      = '6.04.12';
-	$Zanas_VERSION_NAME = $Zanas::VERSION_NAME = 'Space';
+	$Zanas_VERSION      = $Zanas::VERSION      = '6.05.02';
+	$Zanas_VERSION_NAME = $Zanas::VERSION_NAME = 'EcorchE';
 	
 	eval {
 		require Storable;
 	};
 	
 	unless ($preconf -> {core_path}) {
-		require Zanas::Presentation::MSIE_5;
 		require Zanas::Apache;
 		require Zanas::Content;
 		require Zanas::Validators;
@@ -135,6 +137,11 @@ BEGIN {
 		}
 				
 	}
+	
+	$conf -> {skins} ||= ['Classic', 'Dumper', 'XL'];
+	foreach my $skin (@{$conf -> {skins}}) {
+		eval "require Zanas::Presentation::Skins::$skin"
+	}
 
 	if ($Apache::VERSION) {
 		Apache -> push_handlers (PerlChildInitHandler => \&sql_reconnect );
