@@ -61,11 +61,17 @@ sub handler {
 	$_REQUEST {__uri} =~ s{\&salt\=[\d\.]+}{}gsm;
 	
 	our $_SKIN = 'Zanas::Presentation::Skins::' . get_skin_name ();	
+	eval "require $_SKIN";
+	$_SKIN -> {options} ||= $_SKIN -> options;
 	*{$_SKIN . '::_REQUEST'} = *{$_PACKAGE . '_REQUEST'};
+	*{$_SKIN . '::_USER'}    = *{$_PACKAGE . '_USER'};
+	*{$_SKIN . '::Zanas_VERSION_NAME'}    = *{$_PACKAGE . 'Zanas_VERSION_NAME'};
+	*{$_SKIN . '::SQL_VERSION'}    = *{$_PACKAGE . 'SQL_VERSION'};
 	*{$_SKIN . '::conf'    } = *{$_PACKAGE . 'conf'};
 	*{$_SKIN . '::preconf' } = *{$_PACKAGE . 'preconf'};
 	*{$_SKIN . '::r'       } = *{$_PACKAGE . 'r'};
 	*{$_SKIN . '::i18n'    } = *{$_PACKAGE . 'i18n'};
+	*{$_SKIN . '::create_url'    } = *{$_PACKAGE . 'create_url'};
 
 	if ($r -> uri =~ m{/\w+\.(css|gif|ico|js|html)$}) {
 		select__static_files ();
@@ -154,6 +160,16 @@ EOH
 			menu => call_for_role ('select_menu') || call_for_role ('get_menu'),
 			type => $_REQUEST {type},
 		};
+		
+		if ($conf -> {core_extensible_menu} && $_USER -> {systems}) {
+			
+			foreach my $sys (sort grep {/\w/} split /\,/, $_USER -> {systems}) {
+				my @items = ();
+				eval {@items = &{"_${sys}_menu"}()};
+				push @{$page -> {menu}}, @items;
+			}
+
+		}
 
 		call_for_role ('get_page');
 
@@ -320,82 +336,56 @@ sub out_html {
 	
 	return if $_REQUEST {__response_sent};
 	
-	if ($_REQUEST {dbf}) {
-		redirect ("/$html");
+	if ($conf -> {core_sweep_spaces}) {
+		$html =~ s{^\s+}{}gsm; 
+		$html =~ s{[ \t]+}{ }g;
 	}
+
+	unless ($preconf -> {core_no_morons}) {
+		$html =~ s{window\.open}{nope}gsm; 
+	}
+
+
+	$_REQUEST {__content_type} ||= 'text/html; charset=' . $i18n -> {_charset};
+
+	$r -> content_type ($_REQUEST {__content_type});
+	$r -> header_out ('X-Powered-By' => 'Zanas/' . $Zanas::VERSION);
+
+	if ($] > 5.007) {
+		require Encode;
+		$html = Encode::encode ('windows-1252', $html);
+	}
+
+	$preconf -> {core_mtu} ||= 1500;
 	
-	if ($_REQUEST {xls}) {
+	if (
+		($conf -> {core_gzip} or $preconf -> {core_gzip}) && 
+		400 + length $html > $preconf -> {core_mtu} &&
+		($r -> header_in ('Accept-Encoding') =~ /gzip/)
+	) {
+		$r -> content_encoding ('gzip');
+		unless ($_REQUEST {__is_gzipped}) {
+			$html = Compress::Zlib::memGzip ($html);
+		}
+	}
+
+	$r -> header_out ('Content-Length' => length $html);
+
+	if ($preconf -> {core_auth_cookie}) {
 	
-		my $fn_local = '/i/xls/' . time . "$$.xls";
-		my $fn = $r -> document_root . $fn_local;
-		open (O, ">$fn") or die "Can't write to $fn: $!";
-		print O $html;
-		close (O);
+		set_cookie (
+			-name    =>  'sid',
+			-value   =>  $_REQUEST {sid} || 0,
+			-expires =>  $preconf -> {core_auth_cookie},
+			-path    =>  '/',
+		)      
 		
-		download_file ({
-			path => $fn_local,
-			file_name => "file.xls",
-		});
-		
-		unlink $fn;
-		
-	}	
-	else {
+	}
+
+
+	$r -> send_http_header;
 	
-		if ($conf -> {core_sweep_spaces}) {
-			$html =~ s{^\s+}{}gsm; 
-			$html =~ s{[ \t]+}{ }g;
-		}
-
-		unless ($preconf -> {core_no_morons}) {
-			$html =~ s{window\.open}{nope}gsm; 
-		}
-
-
-		$_REQUEST {__content_type} ||= 'text/html; charset=' . $i18n -> {_charset};
-#		$_REQUEST {__content_type} ||= 'application/hta';
-
-		$r -> content_type ($_REQUEST {__content_type});
-		$r -> header_out ('X-Powered-By' => 'Zanas/' . $Zanas::VERSION);
-
-		if ($] > 5.007) {
-			require Encode;
-			$html = Encode::encode ('windows-1252', $html);
-		}
-
-		$preconf -> {core_mtu} ||= 1500;
-		
-		if (
-			($conf -> {core_gzip} or $preconf -> {core_gzip}) && 
-			400 + length $html > $preconf -> {core_mtu} &&
-			($r -> header_in ('Accept-Encoding') =~ /gzip/)
-		) {
-			$r -> content_encoding ('gzip');
-			unless ($_REQUEST {__is_gzipped}) {
-				$html = Compress::Zlib::memGzip ($html);
-			}
-		}
-
-		$r -> header_out ('Content-Length' => length $html);
-#		$r -> header_out ('Set-Cookie' => "sid=$_REQUEST{sid};path=/;") if $_REQUEST{sid};
-
-		if ($preconf -> {core_auth_cookie}) {
-		
-			set_cookie (
-				-name    =>  'sid',
-				-value   =>  $_REQUEST {sid} || 0,
-				-expires =>  $preconf -> {core_auth_cookie},
-				-path    =>  '/',
-			)      
-			
-		}
-
-
-		$r -> send_http_header;
-		
-		$r -> header_only or print $html;
-		
-	}	
+	$r -> header_only or print $html;		
 
 }
 
