@@ -171,7 +171,12 @@ sub peer_query {
 
 sub get_skin_name {
 
-	return 'Classic';
+	return
+		$_REQUEST {xls} ? 'XL' :
+		$_REQUEST {__dump} || $_REQUEST {__d} ? 'Dumper' :
+		$_REQUEST {__proto} ? 'XMLProto' :
+		$_REQUEST {__x} ? 'XMLDumper' :
+		'Classic';
 
 }
 
@@ -682,6 +687,7 @@ sub do_delete_DEFAULT {
 sub do_undelete_DEFAULT {
 
 	sql_do ("UPDATE $_REQUEST{type} SET fake =  0 WHERE id = ?", $_REQUEST{id});
+	sql_undo_relink ($_REQUEST{type}, $_REQUEST{id});
 
 }
 
@@ -995,7 +1001,7 @@ sub redirect {
 		<script src="${root}navigation_${static_salt}.js">
 		</script>
 	</head>
-	<body onLoad="$$options{before}; window.open ('$url&salt=' + Math.random (), $target)">
+	<body onLoad="$$options{before}; nope ('$url&salt=' + Math.random (), $target)">
 	</body>
 </html>
 EOH
@@ -1358,6 +1364,59 @@ sub select__info {
 
 ################################################################################
 
+sub get_item_of__object_info {
+
+	$_REQUEST {__read_only} = 1;
+
+	my $item = sql_select_hash ($_REQUEST {object_type});
+
+	$item -> {last_create} = sql_select_hash ("SELECT * FROM log WHERE type = ? AND action = 'create' AND id_object = ? ORDER BY id DESC LIMIT 1", $_REQUEST {object_type}, $_REQUEST {id});
+	$item -> {last_create} -> {dt} =~ s{(\d+)\-(\d+)\-(\d+)}{$3.$2.$1};
+	$item -> {last_create} -> {user} = sql_select_hash ('users', $item -> {last_create} -> {id_user}); 
+	
+	$item -> {last_update} = sql_select_hash ("SELECT * FROM log WHERE type = ? AND action = 'update' AND id_object = ? ORDER BY id DESC LIMIT 1", $_REQUEST {object_type}, $_REQUEST {id});
+	$item -> {last_update} -> {dt} =~ s{(\d+)\-(\d+)\-(\d+)}{$3.$2.$1};
+	$item -> {last_update} -> {user} = sql_select_hash ('users', $item -> {last_update} -> {id_user}); 
+	
+	my @references = ();
+	
+	foreach my $reference ( sort {$a -> {table_name} . ' ' . $a -> {name} cmp $b -> {table_name} . ' ' . $b -> {name}} @{$DB_MODEL -> {tables} -> {$_REQUEST {object_type}} -> {references}}) {
+
+		my $where = ' WHERE fake = 0 AND ' . $reference -> {name};
+
+		if ($reference -> {TYPE_NAME} =~ /int/) {
+			$where .= " = $_REQUEST{id}";
+		}
+		else {
+			$where .= " LIKE '\%,$_REQUEST{id},\%'";
+		}
+		
+		my $cnt = sql_select_scalar ("SELECT COUNT(*) FROM " . $reference -> {table_name} . $where) or next;
+
+		push @references, {
+			table_name => $reference -> {table_name},
+			name => $reference -> {name},
+			cnt => $cnt,
+		};
+		
+		if ($_REQUEST {table_name} eq $reference -> {table_name} && $_REQUEST {name} eq $reference -> {name}) {
+
+			my $start = $_REQUEST {start} + 0;
+
+			($item -> {records}, $item -> {cnt}) = sql_select_all_cnt ('SELECT * FROM ' . $reference -> {table_name} . $where . " ORDER BY id DESC LIMIT $start, 15");
+
+		}
+		
+	}
+	
+	$item -> {references} = \@references;
+		
+	return $item;
+	
+}
+
+################################################################################
+
 sub get_mac {
 
 	my ($ip) = @_;	
