@@ -38,30 +38,9 @@ sub js_escape {
 sub register_hotkey {
 
 	my ($hashref, $type, $data, $options) = @_;
-
-	$hashref -> {label} =~ s{\&(.)}{<u>$1</u>} or return;
 	
-	my $c = $1;
+	my $code = $_SKIN -> register_hotkey ($hashref) or return;
 	
-	my $code = 0;
-	
-	if ($c eq '<') {
-		$code = 37;
-	}
-	elsif ($c eq '>') {
-		$code = 39;
-	}
-	elsif (lc $c eq '') {
-		$code = 186;
-	}
-	elsif (lc $c eq '') {
-		$code = 222;
-	}
-	else {
-		$c =~ y{}{qwertyuiop[]asdfghjkl;'zxcvbnm,.qwertyuiop[]asdfghjkl;'zxcvbnm,.};
-		$code = (ord ($c) - 32);
-	}
-
 	push @scan2names, {
 		code => $code,
 		type => $type,
@@ -105,13 +84,6 @@ sub hotkey {
 
 ################################################################################
 
-sub dump_attributes {
-	my $attributes = $_[0];	
-	return join ' ', map {"$_='" . $attributes -> {$_} . "'"} keys %$attributes;	
-}
-
-################################################################################
-
 sub trunc_string {
 	my ($s, $len) = @_;
 	return $s if $_REQUEST {xls};
@@ -122,13 +94,8 @@ sub trunc_string {
 
 sub esc_href {
 
-
-#warn ('$conf -> {core_auto_esc} = ' . $conf -> {core_auto_esc} . "\n");
-
 	if ($conf -> {core_auto_esc} == 2) {
 	
-#warn ('esc_href: ' . Dumper (\%REQUEST));
-
 		my $href = sql_select_scalar ('SELECT href FROM __access_log WHERE id_session = ? AND no = ?', $_REQUEST {sid}, $_REQUEST {__last_last_query_string});
 		$href ||= "/?type=$_REQUEST{type}";
 	
@@ -140,8 +107,6 @@ sub esc_href {
 			$href .= '&__scrollable_table_row=' . $_REQUEST {__last_scrollable_table_row};
 		}
 
-#warn ("\$href = '$href'\n");
-
 		return $href;
 		
 	}
@@ -364,7 +329,7 @@ sub draw__info {
 	draw_table (
 	
 		sub {
-			draw_text_cells ({}, [
+			draw_cells ({}, [
 				$i -> {id},
 				{label => $i -> {label}, max_len => 10000000},
 				{label => $i -> {path}, max_len => 10000000},
@@ -429,7 +394,7 @@ sub draw__boot {
 			
 		}					
 						
-		window.open ('$_REQUEST{__uri}?type=logon&redirect_params=$_REQUEST{redirect_params}', '_self');
+		nope ('$_REQUEST{__uri}?type=logon&redirect_params=$_REQUEST{redirect_params}', '_self');
 
 		setTimeout ("document.getElementById ('abuse_1').style.display = 'block'", 10000);
 		
@@ -462,7 +427,7 @@ EOJS
 		
 					<b>!</b>         ,     . ,     ,       Internet: ,   . .
 
-					<p>,        window.open()   $ENV{HTTP_HOST}.
+					<p>,        window . open()   $ENV{HTTP_HOST}.
 			
 				</table>
 			</table>
@@ -471,13 +436,158 @@ EOH
 
 }
 
+################################################################################
+
+sub draw_item_of__object_info {
+
+	my ($data) = @_;
+
+	draw_form (
+	
+		{
+			no_edit => 1,
+		},
+		
+		$data,
+		
+		[
+			[
+				{
+					label => 'id',
+					value => $_REQUEST {id},
+				},
+				{
+					label => 'type',
+					value => $_REQUEST {object_type},
+				},
+			],
+			[
+				{
+					label => 'label',
+					value => $data -> {label},
+				},
+				{
+					label => 'fake',
+					value => $data -> {fake},
+				},
+			],
+			
+			{type => 'banner', label => 'LOG'},
+			
+			[
+				{
+					label  => 'when created',
+					value  => $data -> {last_create} -> {dt},
+					href   => "/?type=log&__popup=1&id=" . $data -> {last_create} -> {id},
+					target => '_blank',
+				},
+				{
+					label => 'when updated',
+					value => $data -> {last_update} -> {dt},
+					href  => "/?type=log&__popup=1&id=" . $data -> {last_update} -> {id},
+					target => '_blank',
+				},
+			],
+			[
+				{
+					label  => 'who created',
+					value  => $data -> {last_create} -> {user} -> {label},
+					href   => "/?type=users&id=" . $data -> {last_create} -> {id_user},
+				},
+				{
+					label  => 'who updated',
+					value  => $data -> {last_update} -> {user} -> {label},
+					href   => "/?type=users&id=" . $data -> {last_update} -> {id_user},
+				},
+			],
+			
+		],
+	
+	)
+	
+	.
+	
+	draw_table (
+	
+		[
+			'table',
+			'column',
+			'count',
+		],
+		
+		sub {
+		
+			draw_cells ({
+				href => {table_name => $i -> {table_name}, name => $i -> {name}},
+			}, [
+				$i -> {table_name},
+				$i -> {name},
+				{
+					label   => $i -> {cnt},
+					picture => '### ### ### ### ###',
+					off     => 'if zero',
+				},
+			])
+		
+		},
+		
+		$data -> {references},
+		
+		{
+			title => {label => 'References'},
+			lpt => 1,
+		},
+	
+	)
+	
+	.
+	
+	draw_table (
+	
+		[
+			'id',
+			'label',
+			'dt',
+		],
+	
+		sub {
+			
+			$i -> {dt} =~ s{(\d+)\-(\d+)\-(\d+)}{$3.$2.$1};
+		
+			draw_cells ({
+				href => "/?type=$_REQUEST{table_name}&id=$$i{id}",
+			}, [
+				$i -> {id},
+				$i -> {label} || $i -> {no},
+				$i -> {dt},
+			])
+		
+		},
+		
+		$data -> {records},
+		
+		{
+			title => {label => "Referring $_REQUEST{table_name} by $_REQUEST{name}"},
+			off => !$_REQUEST {table_name},
+			top_toolbar => [{}, {
+				type => 'pager',
+				cnt  => 0 + @{$data -> {records}},
+				total => $data -> {cnt},
+			}],
+			
+		},
+	
+	)
+	
+}
+
 ################################################################################
 #                     R   E   F   A   C   T   O   R   E   D                    #
 ################################################################################
 
 sub draw_auth_toolbar {
 
-	$_REQUEST {__no_navigation} and return '';
+	return '' if $_REQUEST {__no_navigation} or $conf -> {core_no_auth_toolbar};
 	
 	return $_SKIN -> draw_auth_toolbar ({
 		top_toolbar => ($conf -> {top_banner} ? interpolate ($conf -> {top_banner}) : ''),
@@ -492,9 +602,7 @@ sub draw_auth_toolbar {
 sub draw_hr {
 
 	my (%options) = @_;
-	
-	return '' if $_REQUEST {xls};
-	
+		
 	$options {height} ||= 1;
 	$options {class}  ||= bgr8;
 	
@@ -509,7 +617,6 @@ sub draw_window_title {
 	my ($options) = @_;
 	
 	return '' if $options -> {off};
-	return '' if $_REQUEST {xls};
 	
 	our $__last_window_title = $options -> {label};
 		
@@ -564,15 +671,15 @@ sub draw_form {
 	elsif ($conf -> {core_auto_esc} > 0 && $_REQUEST {__last_query_string}) {
 		$options -> {esc} = esc_href ();
 	}	
-		
-my $proto_path = join ' / ', map {$_ -> {name}} @{$data -> {path}};
-$prototype .= qq{\t<form path="$proto_path">\n};
-				
+						
 	our $tabindex = 1;
 	
 	my @rows = ();
 			
 	foreach my $field (@$fields) {
+		
+		my $row;
+		
 		if (ref $field eq ARRAY) {
 			my @row = ();
 			foreach (@$field) {
@@ -581,13 +688,16 @@ $prototype .= qq{\t<form path="$proto_path">\n};
 				push @row, $_;
 			}
 			next if @row == 0;
-			push @rows, \@row;
+			$row = \@row;
 		}
 		else {
 			next if $field -> {off};
 			next if $_REQUEST {__read_only} && $field -> {type} eq 'password';
-			push @rows, [$field];
-		}		
+			$row = [$field];
+		}
+		
+		push @rows, $row;
+
 	}
 	
 	my $max_colspan = 1;
@@ -604,9 +714,13 @@ $prototype .= qq{\t<form path="$proto_path">\n};
 		$max_colspan > $sum_colspan or $max_colspan = $sum_colspan;
 	}
 
+	$_SKIN -> start_form () if $_SKIN -> {options} -> {no_buffering};
+
 	foreach my $row (@rows) {
 		$row -> [-1] -> {colspan} += ($max_colspan - $row -> [-1] -> {sum_colspan});
+		$_SKIN -> start_form_row () if $_SKIN -> {options} -> {no_buffering};
 		foreach (@$row) { $_ -> {html} = draw_form_field ($_, $data) };
+		$_SKIN -> draw_form_row ($row) if $_SKIN -> {options} -> {no_buffering};
 	}
 	
 	$options -> {rows} = \@rows;
@@ -665,12 +779,8 @@ $prototype .= qq{\t<form path="$proto_path">\n};
 			draw_ok_esc_toolbar ($options, $data);
 
 	}
-
-$prototype .= $__last_centered_toolbar;	
-$__last_centered_toolbar = '';
-$prototype .= "\t</form>\n";
 	
-	$__last_centered_toolbar_id = '';
+#	$__last_centered_toolbar_id = '';
 	
 	return $_SKIN -> draw_form ($options);
 
@@ -820,8 +930,6 @@ sub draw_form_field_string {
 			
 	$options -> {attributes} -> {tabindex} = ++ $_REQUEST {__tabindex};
 
-$prototype .= qq{\t\t\t<input label="$$options{label}" type="text" size="$$options{size}" value="$s" />\n};
-
 	return $_SKIN -> draw_form_field_string (@_);
 	
 }
@@ -870,9 +978,6 @@ sub draw_form_field_datetime {
 
 	$options -> {attributes} -> {tabindex} = ++ $_REQUEST {__tabindex};
 
-my $proto_type = $options -> {no_time} ? 'date' : 'datetime';
-$prototype .= qq{\t\t\t<input label="$$options{label}" type="$proto_type" value="$s" size="$$options{size}" />\n};
-
 	return $_SKIN -> draw_form_field_datetime (@_);
 
 }
@@ -900,8 +1005,6 @@ sub draw_form_field_hgroup {
 
 	my ($options, $data) = @_;
 			
-$prototype .= qq {\t\t<hgroup label="$$options{label}">\n};
-			
 	foreach my $item (@{$options -> {items}}) {
 		next if $item -> {off};
 		$item -> {label} .= ': ' if $item -> {label} && !$item -> {no_colon};
@@ -910,8 +1013,6 @@ $prototype .= qq {\t\t<hgroup label="$$options{label}">\n};
 		$item -> {html}   = &{'draw_form_field_' . $item -> {type}} ($item, $data);
 	}
 	
-$prototype .= "\t\t</hgroup>\n";
-
 	return $_SKIN -> draw_form_field_hgroup (@_);
 		
 }
@@ -944,8 +1045,6 @@ sub draw_form_field_password {
 	$options -> {size} ||= $conf -> {size} || 120;	
 	$options -> {attributes} -> {tabindex} = ++ $_REQUEST {__tabindex};
 	
-$prototype .= qq{<input label="$$options{label}" type="password" size="$$options{size}" />\n};
-
 	return $_SKIN -> draw_form_field_password (@_);
 	
 }
@@ -1043,15 +1142,11 @@ sub draw_form_field_checkbox {
 sub draw_form_field_radio {
 
 	my ($options, $data) = @_;
-		
-$prototype .= qq{\t\t\t<input label="$$options{label}" type="radio">\n};
-	
+			
 	$options -> {values} = [ grep { !$_ -> {off} } @{$options -> {values}} ];
 
 	foreach my $value (@{$options -> {values}}) {
 	
-$prototype .= qq{\t\t\t\t<option label="$$value{label}>"};
-
 		$value -> {attributes} -> {tabindex} = ++ $_REQUEST {__tabindex};
 		$value -> {attributes} -> {checked} = 1 if $data -> {$options -> {name}} == $value -> {id};
 					
@@ -1062,13 +1157,9 @@ $prototype .= qq{\t\t\t\t<option label="$$value{label}>"};
 		
 		$value -> {html} = &$renderrer ($value, $data);
 		delete $value -> {attributes} -> {class};
-				
-$prototype .= qq{</option>\n};
-		
+						
 	}
-	
-$prototype .= qq{\t\t\t</input>\n};
-		
+			
 	return $_SKIN -> draw_form_field_radio (@_);
 	
 }
@@ -1204,7 +1295,6 @@ sub draw_toolbar {
 	my ($options, @buttons) = @_;
 	
 	return '' if $options -> {off};	
-	return '' if $_REQUEST {xls};
 	
 	$_REQUEST {__toolbars_number} ||= 0;
 		
@@ -1260,9 +1350,7 @@ sub draw_toolbar_break {
 sub draw_toolbar_button {
 
 	my ($options) = @_;
-		
-$__last_top_toolbar .= qq{\t\t\t<button label="$$options{label}" icon="$$options{icon}" />\n};
-	
+			
 	$conf -> {kb_options_buttons} ||= {ctrl => 1, alt => 1};	
 	
 	register_hotkey ($options, 'href', $options, $conf -> {kb_options_buttons});
@@ -1290,7 +1378,7 @@ $__last_top_toolbar .= qq{\t\t\t<button label="$$options{label}" icon="$$options
 		my $salt = rand;
 		my $msg = js_escape ($options -> {confirm});
 		$options -> {href} =~ s{\%}{\%25}gsm; 		# wrong, but MSIE uri_unescapes the 1st arg of window.open :-(
-		$options -> {href} = qq [javascript:if (confirm ($msg)) {window.open('$$options{href}', '$$options{target}')}];
+		$options -> {href} = qq [javascript:if (confirm ($msg)) {nope('$$options{href}', '$$options{target}')}];
 	} 
 	
 	if ($options -> {href} =~ /^java/) {
@@ -1316,8 +1404,6 @@ sub draw_toolbar_input_select {
 
 	my ($options) = @_;
 		
-$__last_top_toolbar .= qq{\t\t\t<input type="select" label="$$options{label}" />\n};
-
 	$options -> {max_len} ||= $conf -> {max_len};
 	
 	foreach my $value (@{$options -> {values}}) {		
@@ -1508,7 +1594,7 @@ sub draw_centered_toolbar_button {
 		my $msg = js_escape ($options -> {confirm});
 		$options -> {preconfirm} ||= 1;
 		$options -> {href} =~ s{\%}{\%25}gsm; 		# wrong, but MSIE uri_unescapes the 1st arg of window.open :-(
-		$options -> {href} = qq [javascript:if (!($$options{preconfirm}) || ($$options{preconfirm} && confirm ($msg))) {window.open('$$options{href}', '$target')}];
+		$options -> {href} = qq [javascript:if (!($$options{preconfirm}) || ($$options{preconfirm} && confirm ($msg))) {nope('$$options{href}', '$target')} else {window.parent.document.body.style.cursor = 'normal'; nop ();} ];
 	} 	
 
 	if ($options -> {href} =~ /^java/) {
@@ -1527,9 +1613,7 @@ sub draw_centered_toolbar {
 
 	my ($options, $list) = @_;
 
-our $__last_centered_toolbar = "\t\t<panel>\n";
-
-our $__last_centered_toolbar_id = 'toolbar_' . int $list;
+#	our $__last_centered_toolbar_id = 'toolbar_' . int $list;
 
 	$options -> {cnt} = 0;
 	
@@ -1539,8 +1623,6 @@ our $__last_centered_toolbar_id = 'toolbar_' . int $list;
 		$options -> {cnt} ++;
 	}
 
-$__last_centered_toolbar .= "\t\t</panel>\n";
-
 	return $_SKIN -> draw_centered_toolbar (@_);
 
 }
@@ -1677,20 +1759,31 @@ sub draw_menu {
 	
 	$_REQUEST {__no_navigation} and return '';	
 
-	if ($conf -> {core_show_dump} || $preconf -> {core_show_dump}) {
+	if ($preconf -> {core_show_dump}) {
 	
 		push @$types, {
-			label => 'Dump',
-			name  => '_dump',
-			href  => create_url () . '&__dump=1',
-			side  => 'right_items',
+			label  => 'Dump',
+			name   => '_dump',
+			href   => create_url () . '&__dump=1',
+			side   => 'right_items',
+			target => '_blank',
+			no_off => 1,
 		};
 
 		push @$types, {
-			label => 'Proto',
-			name  => '_proto',
-			href  => create_url () . '&__proto=1',			
-			side  => 'right_items',
+			label  => 'Info',
+			href   => "/?type=_object_info&object_type=$_REQUEST{type}&id=$_REQUEST{id}",
+			side   => 'right_items',
+			no_off => 1,
+		} if $_REQUEST {id} && $DB_MODEL -> {tables} -> {$_REQUEST {type}};
+
+		push @$types, {
+			label  => 'Proto',
+			name   => '_proto',
+			href   => create_url () . '&__proto=1&__edit=' . $_REQUEST {__edit},			
+			side   => 'right_items',
+			target => '_blank',
+			no_off => 1,
 		};
 	
 	}
@@ -1722,7 +1815,7 @@ sub draw_menu {
 	
 		register_hotkey ($type, 'href', 'main_menu_' . $type -> {name}, $conf -> {kb_options_menu});
 		
-		if ($_REQUEST {__edit}) {
+		if ($_REQUEST {__edit} && !$type -> {no_off}) {
 			$type -> {href} = "javaScript:alert('$$i18n{save_or_cancel}'); document.body.style.cursor = 'normal'; nop ();";
 		}
 		elsif ($type -> {no_page}) {
@@ -1735,11 +1828,20 @@ sub draw_menu {
 		}
 
 		if (ref $type -> {items} eq ARRAY && !$_REQUEST {__edit}) {
-			$type -> {vert_menu} .= draw_vert_menu ($type -> {name}, $type -> {items}, $type -> {label});
+			$type -> {vert_menu} = draw_vert_menu ($type -> {name}, $type -> {items});
 			$type -> {onhover} = "open_popup_menu ('$$type{name}')";
 		}
 		else {
-			$type -> {onhover} = "if (last_vert_menu) {last_vert_menu.style.display = 'none'; last_vert_menu = null}";
+			$type -> {onhover} = <<EOJS
+	if (last_vert_menu [0]) {
+	
+		for (var i = 0; i < last_vert_menu.length; i++) {
+			if (last_vert_menu [i]) last_vert_menu [i].style.display = 'none';
+			last_vert_menu [i] = null;
+		}
+		
+	};
+EOJS
 		}
 		
 		$type -> {side  } ||= 'left_items';
@@ -1757,19 +1859,37 @@ sub draw_menu {
 
 sub draw_vert_menu {
 
-	my ($name, $types) = @_;
+	my ($name, $types, $level) = @_;
+	
+	$level ||= 0;
+	
+	$types = [grep {!$_ -> {off}} @$types];
 	
 	foreach my $type (@$types) {
 	
-			$type -> {href} ||= "/?type=$$type{name}";
-			$type -> {href} .= "&role=$$type{role}" if $type -> {role};
-			check_href ($type);	
+		if (ref $type -> {items} eq ARRAY && !$_REQUEST {__edit}) {
+
+			my $sublevel = $level + 1;
+			$type -> {name}     ||= '' . $type if $type -> {items};
+			$type -> {vert_menu}  = draw_vert_menu ($type -> {name}, $type -> {items}, $sublevel);
+			$type -> {onclick}    = "open_popup_menu ('$$type{name}', $sublevel)";
 			
+		}
+		else {
+			
+			$type -> {onhover}    = '';
+			$type -> {href}     ||= "/?type=$$type{name}";
+			$type -> {href}      .= "&role=$$type{role}" if $type -> {role};
+
+			check_href ($type);
+
 			$type -> {onclick} = $type -> {href} =~ /^javascript\:/i ? $' : "activate_link('$$type{href}')";  #'
 			$type -> {onclick} =~ s{[\n\r]}{}gsm;
+		}
+	
 	}
 
-	return $_SKIN -> draw_vert_menu (@_);
+	return $_SKIN -> draw_vert_menu ($name, $types, $level);
 
 }
 
@@ -1790,12 +1910,12 @@ sub draw_cells {
 	foreach my $cell (@{$_[0]}) {
 	
 		$result .= 
-			!ref ($cell)                  ? draw_text_cell ($cell, $options) :
-			$cell -> {type}  eq 'radio'   ? draw_radio_cell ($cell, $options) :
-			$cell  -> {type} eq 'select'   ? draw_select_cell ($cell, $options) :
+			!ref ($cell) || $cell -> {off} || $cell -> {read_only} ? draw_text_cell ($cell, $options) :
 			($cell -> {type} eq 'checkbox' || exists $cell -> {checked}) ? draw_checkbox_cell ($cell, $options) :
-			$cell  -> {type} eq 'input'   ? draw_input_cell ($cell, $options) :
 			($cell -> {type} eq 'button'   || $cell -> {icon}) ? draw_row_button ($cell, $options) :		
+			$cell  -> {type} eq 'input'    ? draw_input_cell  ($cell, $options) :
+			$cell  -> {type} eq 'radio'    ? draw_radio_cell  ($cell, $options) :
+			$cell  -> {type} eq 'select'   ? draw_select_cell ($cell, $options) :
 			draw_text_cell ($cell, $options);
 	
 	}
@@ -1807,35 +1927,21 @@ sub draw_cells {
 ################################################################################
 
 sub draw_text_cells {
-
-	my $options = (ref $_[0] eq HASH) ? shift () : {};
-	
-	my $html = '';
-	
-	foreach (@{$_[0]}) { $html .= draw_text_cell ($_, $options) }
-		
-	return $html;
-
+	return draw_cells (@_);	
 }
 
 ################################################################################
 
-sub draw_text_cell {
+sub draw_row_buttons {
+	return draw_cells (@_);	
+}
+
+################################################################################
+
+sub _adjust_row_cell_style {
 
 	my ($data, $options) = @_;
-	
-	return '' if ref $data eq HASH && $data -> {hidden};
 
-	ref $data eq HASH or $data = {label => $data};
-		
-if ($i -> {__n} == 0) {
-	my $__header = $__headers -> [$__last_header_id ++];
-	$__header = $__header -> {label} if ref $__header eq HASH;
-	$__header =~ s{\<.*?\>}{}gsm;
-	$__header =~ s{\w\w\w\w;}{}gsm;
-$prototype .= qq{\t\t\t<column label="$__header" text="$$data{label}" />\n};
-}
-	
 	$data -> {attributes} ||= {};
 	$data -> {attributes} -> {colspan} = $data -> {colspan} if $data -> {colspan};
 	$data -> {attributes} -> {rowspan} = $data -> {rowspan} if $data -> {rowspan};
@@ -1850,7 +1956,25 @@ $prototype .= qq{\t\t\t<column label="$__header" text="$$data{label}" />\n};
 		delete $data -> {attributes} -> {style};
 		$data -> {attributes} -> {class} ||= $data    -> {class};
 		$data -> {attributes} -> {class} ||= $options -> {class};
+		$data -> {attributes} -> {class} ||= 
+			$options -> {is_total} ? 'row-cell-total' : 
+			$data -> {attributes} -> {bgcolor} ? 'row-cell-transparent' : 
+			'row-cell';
 	}	
+
+}
+
+################################################################################
+
+sub draw_text_cell {
+
+	my ($data, $options) = @_;
+	
+	return '' if ref $data eq HASH && $data -> {hidden};
+
+	ref $data eq HASH or $data = {label => $data};
+			
+	_adjust_row_cell_style ($data, $options);
 				
 	$data -> {off} = is_off ($data, $data -> {label});
 	
@@ -1905,4 +2029,432 @@ $prototype .= qq{\t\t\t<column label="$__header" text="$$data{label}" />\n};
 
 }
 
+################################################################################
+
+sub draw_radio_cell {
+
+	my ($data, $options) = @_;
+	
+	return draw_text_cell (@_) if $data -> {read_only} || $data -> {off};
+	
+	$data -> {value} ||= 1;	
+	$data -> {checked} = $data -> {checked} ? 'checked' : '';
+
+	_adjust_row_cell_style ($data, $options);
+
+	check_title ($data);
+	
+	return $_SKIN -> draw_radio_cell ($data, $options);
+
+}
+
+################################################################################
+
+sub draw_checkbox_cell {
+
+	my ($data, $options) = @_;
+	
+	return draw_text_cell (@_) if $data -> {read_only} || $data -> {off};
+
+	$data -> {value} ||= 1;	
+	$data -> {checked} = $data -> {checked} ? 'checked' : '';
+
+	_adjust_row_cell_style ($data, $options);
+
+	check_title ($data);
+
+	return $_SKIN -> draw_checkbox_cell ($data, $options);
+	
+}
+
+################################################################################
+
+sub draw_select_cell {
+
+	my ($data, $options) = @_;
+
+	return draw_text_cell ($data, $options) if ($_REQUEST {__read_only} && !$data -> {edit}) || $data -> {read_only} || $data -> {off};
+	
+	$data -> {max_len} ||= $conf -> {max_len};
+
+	_adjust_row_cell_style ($data, $options);
+
+	foreach my $value (@{$data -> {values}}) {
+		$value -> {selected} = ($value -> {id} eq $data -> {value}) ? 'selected' : '';
+		$value -> {label} = trunc_string ($value -> {label}, $data -> {max_len});
+		$value -> {id} =~ s{\"}{\&quot;}g; #"
+	}
+
+	return $_SKIN -> draw_select_cell ($data, $options);
+	
+}
+
+################################################################################
+
+sub draw_input_cell {
+
+	my ($data, $options) = @_;
+	
+	return draw_text_cell ($data, $options) if ($_REQUEST {__read_only} && !$data -> {edit}) || $data -> {read_only} || $data -> {off};
+
+	$data -> {size} ||= 30;
+
+	$data -> {attributes} ||= {};
+	$data -> {attributes} -> {class} ||= 'row-cell';
+	
+	_adjust_row_cell_style ($data, $options);
+						
+	$data -> {label} ||= '';
+	
+	if ($data -> {picture}) {
+		$data -> {label} = format_picture ($data -> {label}, $data -> {picture});
+		$data -> {label} =~ s/^\s+//g;
+		$data -> {attributes} -> {align} ||= 'right';
+	}
+			
+	check_title ($data);
+		
+	return $_SKIN -> draw_input_cell ($data, $options);
+
+}
+
+################################################################################
+
+sub draw_row_button {
+
+	my ($options) = @_;	
+	
+	check_href  ($options);
+
+	if ($options -> {confirm}) {
+		my $salt = rand;
+		my $msg = js_escape ($options -> {confirm});
+		$options -> {href} =~ s{\%}{\%25}gsm; 		# wrong, but MSIE uri_unescapes the 1st arg of window.open :-(
+		$options -> {href} = qq [javascript:if (confirm ($msg)) {nope('$$options{href}', '_self')} else {document.body.style.cursor = 'normal'; nop ();}];
+	}
+
+	if (
+		$conf -> {core_auto_esc} == 2 && 
+		! (	
+			$options -> {keep_esc} ||
+			(!exists $options -> {keep_esc} && $options -> {icon} eq 'delete' && !$_REQUEST {id})
+		)
+
+	) {
+		$options -> {href} =~ s{__last_query_string\=\d+}{__last_query_string\=$_REQUEST{__last_last_query_string}}gsm;
+	}
+
+	if ($options -> {href} =~ /^java/) {
+		$options -> {target} = '_self';
+	}
+
+	check_title ($options);
+
+	return $_SKIN -> draw_row_button ($options);
+
+}
+
+################################################################################
+
+sub draw_table_header {
+
+	my ($rows) = @_;
+	
+	ref $rows -> [0] eq ARRAY or $rows = [$rows];
+	
+	return $_SKIN -> draw_table_header ($rows, [map {draw_table_header_row ($_)} @$rows]);
+	
+}
+
+################################################################################
+
+sub draw_table_header_row {
+
+	my ($cells) = @_;
+		
+	return $_SKIN -> draw_table_header_row ($rows, [map {
+		ref $_ eq ARRAY ? (join map {draw_table_header_cell ($_)} @$_) : draw_table_header_cell ($_)
+	} @$cells]);
+	
+}
+
+################################################################################
+
+sub draw_table_header_cell {
+
+	my ($cell) = @_;
+	
+	ref $cell eq HASH or $cell = {label => $cell};
+
+	check_title ($cell);
+	
+	foreach my $field (qw(href href_asc href_desc)) {
+	
+		$cell -> {$field} or next;
+		
+		my $h = {href => $cell -> {$field}};
+		check_href ($h);
+		$cell -> {$field} => $h -> {href};
+	
+	}
+	
+	$cell -> {colspan} ||= 1;
+	$cell -> {rowspan} ||= 1;
+	
+	$cell -> {attributes} ||= {};
+	$cell -> {attributes} -> {class}   ||= 'row-cell-header';
+	$cell -> {attributes} -> {colspan} ||= $cell -> {colspan};
+	$cell -> {attributes} -> {rowspan} ||= $cell -> {rowspan};
+	
+	return $_SKIN -> draw_table_header_cell ($cell);
+
+}
+
+################################################################################
+
+sub draw_table_row {
+
+	my ($n, $tr_callback) = @_;
+
+	$i -> {__n} = $n;		
+	$i -> {__types} = [];
+	$i -> {__trs}   = [];
+	
+	$_SKIN -> {__current_row} = $i;
+
+	my $tr_id = {href => 'id=' . $i -> {id}};
+	check_href ($tr_id);
+	$tr_id -> {href} =~ s{\&salt=[\d\.]+}{};
+	$i -> {__tr_id} = $tr_id -> {href};
+
+	foreach my $callback (@$tr_callback) {
+
+		our $_FLAG_ADD_LAST_QUERY_STRING = 1;
+
+		$_SKIN -> start_table_row if $_SKIN -> {options} -> {no_buffering};
+		my $tr = &$callback ();
+		$_SKIN -> draw_table_row ($tr) if $_SKIN -> {options} -> {no_buffering};
+					
+		undef $_FLAG_ADD_LAST_QUERY_STRING;
+		
+		$tr or next;
+		
+		$scrollable_row_id ++;
+		
+		push @{$i -> {__trs}}, $tr unless $_SKIN -> {options} -> {no_buffering};
+					
+	}
+	
+	if (@{$i -> {__types}} > 0) {			
+		$i -> {__menu} = draw_vert_menu ($i, $i -> {__types});			
+	}
+
+}
+
+################################################################################
+
+sub draw_table {
+
+	return '' if $_REQUEST {__only_form};
+
+	my $headers = [];
+
+	unless (ref $_[0] eq CODE or (ref $_[0] eq ARRAY and ref $_[0] -> [0] eq CODE)) {
+		$headers = shift;
+	}
+
+	my ($tr_callback, $list, $options) = @_;
+	
+	$options -> {type}   ||= $_REQUEST{type};
+	
+	$options -> {action} ||= 'add';
+	$options -> {name}   ||= 'form';
+	$options -> {header}   = draw_table_header ($headers) if @$headers > 0;
+
+	return '' if $options -> {off};		
+
+	ref $tr_callback eq ARRAY or $tr_callback = [$tr_callback];
+		
+	if (ref $options -> {title} eq HASH) {
+				
+		unless ($_REQUEST {select}) {
+			$options -> {title} -> {height} ||= 10;
+			$options -> {title} = 
+				draw_hr (%{$options -> {title}}) .
+				draw_window_title ($options -> {title}) if $options -> {title} -> {label};
+		}
+		
+	}
+	
+	if (ref $options -> {top_toolbar} eq ARRAY) {			
+		$options -> {top_toolbar} = draw_toolbar (@{ $options -> {top_toolbar} });
+	}
+	
+	if (ref $options -> {path} eq ARRAY) {
+		$options -> {path} = draw_path ({}, $options -> {path});
+	}
+	
+	if ($options -> {'..'} && !$_REQUEST{lpt}) {
+	
+		my $url = $_REQUEST {__path} -> [-1];
+		if ($conf -> {core_auto_esc} > 0 && $_REQUEST {__last_query_string}) {
+			$url = esc_href ();
+		}
+
+		our $_FLAG_ADD_LAST_QUERY_STRING = 1;
+
+		$options -> {dotdot} = draw_text_cell ({
+			a_id  => 'dotdot',
+			label => '..',
+			href  => $url,
+			no_select_href => 1,
+			colspan => 0 + @$headers,
+		});
+		
+		$scrollable_row_id ++;
+		
+		undef $_FLAG_ADD_LAST_QUERY_STRING;
+
+		hotkey ({code => Esc, data => 'dotdot'});
+
+	}
+
+	$_SKIN -> start_table ($options) if $_SKIN -> {options} -> {no_buffering};
+
+	my $n = 1;
+	
+	if (ref $list eq 'DBI::st') {
+	
+		while (our $i = $list -> fetchrow_hashref) {
+			draw_table_row ($n++, $tr_callback);
+		}
+		
+		$list -> finish;
+		
+	}
+	else {
+
+		foreach our $i (@$list) {
+			draw_table_row ($n++, $tr_callback);
+		}
+		
+	}
+	
+	my $html = $_SKIN -> draw_table ($tr_callback, $list, $options);
+	
+	$lpt = 1 if $options -> {lpt};
+	
+	return $html;
+
+}
+
+################################################################################
+
+sub draw_tr {
+
+	### O B S O L E T E !!!
+	
+	my ($options, @tds) = @_;	
+	return qq {<tr>@tds</tr>};
+
+}
+
+################################################################################
+
+sub draw_one_cell_table {
+
+	### A B A N D O N E D !!!
+
+	my ($options, $body) = @_;
+	return $_SKIN -> draw_one_cell_table ($options, $body);
+
+}
+
+################################################################################
+
+sub draw_page {
+
+	my ($page) = @_;
+	
+	$_REQUEST {lpt} ||= $_REQUEST {xls};
+	$_REQUEST {__read_only} = 1 if ($_REQUEST {lpt});
+		
+	delete $_REQUEST {__response_sent};
+	
+	$page -> {body} = '';
+
+	my ($selector, $renderrer);
+	
+	unless ($_REQUEST {error}) {
+	
+		if ($_REQUEST {id}) {
+			$selector  = 'get_item_of_' . $page -> {type};
+			$renderrer = 'draw_item_of_' . $page -> {type};
+		} 
+		elsif ($_REQUEST {dbf}) {
+			$selector  = 'select_' . $page -> {type};
+			$renderrer = 'dbf_write_' . $page -> {type};
+		} 
+		else {
+			$selector  = 'select_' . $page -> {type};
+			$renderrer = 'draw_' . $page -> {type};
+		}
+		
+		undef $page -> {content};		
+		
+		eval { $page -> {content} = call_for_role ($selector)};
+
+		if ($@) {
+			warn $@;
+			$_REQUEST {error} = $@;
+		}
+		
+		return '' if $_REQUEST {__response_sent};
+		
+		return $_SKIN -> draw_page ($page) if !$_REQUEST {error} && $_SKIN -> {options} -> {no_presentation};
+
+		if ($conf -> {core_auto_edit} && $_REQUEST {id} && ref $page -> {content} eq HASH && $page -> {content} -> {fake} > 0) {
+			$_REQUEST {__edit} = 1;
+		}
+						
+		if ($_REQUEST {__popup}) {
+			$_REQUEST {__read_only} = 1;
+			$_REQUEST {__pack} = 1;
+			$_REQUEST {__no_navigation} = 1;
+		}
+		
+		our @scan2names = ();	
+		our $scrollable_row_id = 0;
+		our $lpt = 0;
+
+		eval { 
+			$_SKIN -> start_page ($page) if $_SKIN -> {options} -> {no_buffering};
+			$page  -> {auth_toolbar} = draw_auth_toolbar ();
+			$page  -> {body} 	= call_for_role ($renderrer, $page -> {content}); 
+			$page  -> {menu}         = draw_menu ($page -> {menu}, $page -> {highlighted_type}, {lpt => $lpt});
+		};
+		
+		$page -> {scan2names} = \@scan2names;
+		
+		if ($@) {
+			warn $@;
+			$_REQUEST {error} = $@;
+		}
+						
+	}
+
+	if ($_REQUEST {error}) {
+		
+		if ($_REQUEST {error} =~ s{^\#(\w+)\#\:}{}) {
+			$page -> {error_field} = $1;
+		}
+
+		return $_SKIN -> draw_error_page ($page);		
+		
+	}
+		
+	return $_SKIN -> draw_page ($page);		
+		
+}
+
 1;
\ No newline at end of file
