@@ -1,9 +1,44 @@
 package Zanas::Presentation::Skins::Classic;
 
-use Zanas::Presentation::Skins::Generic;
+BEGIN {
+	require Zanas::Presentation::Skins::Generic;
+	delete $INC {"Zanas/Presentation/Skins/Generic.pm"};
+}
 
-#use Data::Dumper;
-#$Data::Dumper::Sortkeys = 1;
+################################################################################
+
+sub options {
+	return {};
+}
+
+################################################################################
+
+sub register_hotkey {
+
+	my ($_SKIN, $hashref) = @_;
+
+	$hashref -> {label} =~ s{\&(.)}{<u>$1</u>} or return undef;
+
+	my $c = $1;
+		
+	if ($c eq '<') {
+		return 37;
+	}
+	elsif ($c eq '>') {
+		return 39;
+	}
+	elsif (lc $c eq '') {
+		return 186;
+	}
+	elsif (lc $c eq '') {
+		return 222;
+	}
+	else {
+		$c =~ y{}{qwertyuiop[]asdfghjkl;'zxcvbnm,.qwertyuiop[]asdfghjkl;'zxcvbnm,.};
+		return (ord ($c) - 32);
+	}
+
+}
 
 ################################################################################
 
@@ -628,7 +663,7 @@ sub draw_form_field_select {
 			onKeyDown="tabOnEnter()"
 			onChange="is_dirty=true; $$options{onChange}" 
 			onKeyPress="typeAhead()" 
-			style="visibility:expression(last_vert_menu ? 'hidden' : '')"
+			style="visibility:expression(last_vert_menu [0] ? 'hidden' : '')"
 		>
 EOH
 		
@@ -746,7 +781,7 @@ sub draw_form_field_image {
 	return <<EOH;
 		<input type="hidden" name="_$$options{name}" value="$$options{id_image}">
 		<img src="$$options{src}" id="$$options{name}_preview" width = "$$options{width}" height = "$$options{height}">&nbsp;
-		<input type="button" value="$$i18n{Select}" onClick="window.open('$$options{new_image_url}', 'selectImage' , '');">
+		<input type="button" value="$$i18n{Select}" onClick="nope('$$options{new_image_url}', 'selectImage' , '');">
 EOH
 
 }
@@ -956,7 +991,7 @@ sub draw_toolbar_input_select {
 	}
 	
 	$html .= <<EOH;
-		<select name="$$options{name}" onChange="submit()" onkeypress="typeAhead()" style="visibility:expression(last_vert_menu ? 'hidden' : '')">
+		<select name="$$options{name}" onChange="submit()" onkeypress="typeAhead()" style="visibility:expression(last_vert_menu [0] ? 'hidden' : '')">
 EOH
 
 	if (defined $options -> {empty}) {
@@ -1039,7 +1074,7 @@ sub draw_toolbar_input_text {
 			value="$$options{value}" 
 			onFocus="scrollable_table_is_blocked = true; q_is_focused = true" 
 			onBlur="scrollable_table_is_blocked = false; q_is_focused = false"
-			style="visibility:expression(last_vert_menu ? 'hidden' : '')"
+			style="visibility:expression(last_vert_menu [0] ? 'hidden' : '')"
 		>
 EOH
 
@@ -1146,9 +1181,7 @@ sub draw_centered_toolbar {
 
 	my ($_SKIN, $options, $list) = @_;
 
-our $__last_centered_toolbar = "\t\t<panel>\n";
-
-our $__last_centered_toolbar_id = 'toolbar_' . int $list;
+	our $__last_centered_toolbar_id = 'toolbar_' . int $list;
 	
 	my $colspan = 3 * (1 + $options -> {cnt}) + 1;
 
@@ -1256,10 +1289,13 @@ EOH
 							+ this.offsetParent.offsetTop 
 							+ this.offsetParent.offsetParent.offsetTop 
 							+ 2
-						) && last_vert_menu
+						) && last_vert_menu [0]
 					) {
-						last_vert_menu.style.display = 'none';
-						last_vert_menu = null;
+						
+						for (var i = 0; i < last_vert_menu.length; i++) {
+							if (last_vert_menu [i]) last_vert_menu [i].style.display = 'none';
+						}
+						
 					}				
 				" 
 				class="main-menu" 
@@ -1291,18 +1327,13 @@ EOH
 
 sub draw_vert_menu {
 
-	my ($_SKIN, $name, $types) = @_;
-	
+	my ($_SKIN, $name, $types, $level) = @_;
+		
 	my $html = <<EOH;
 		<div 
 			id="vert_menu_$name" 
 			style="display:none; position:absolute; z-index:100" 
-			onmouseout="
-				if (!hasMouse (this, event)) {
-					this.style.display = 'none';
-					last_vert_menu = null;
-				}				
-			"
+			onmouseout="check_popup_menus(event)"
 		>
 			<table id="vert_menu_table_$name" width=1 bgcolor=#d5d5d5 cellspacing=0 cellpadding=0 border=0 border=1>
 				<tr height=1>
@@ -1328,10 +1359,11 @@ EOH
 					<td bgcolor=#ffffff width=1><img height=2 src=$_REQUEST{__uri}0.gif width=1 border=0></td>
 
 					<td onmouseover="blockEvent ();" onmouseout="blockEvent ();">
-					<table width=90% border=0 cellspacing=0 cellpadding=0 align=center minheight=2>
-						<tr height=1><td bgcolor="#888888"><img height=1 src=$_REQUEST{__uri}0.gif width=1 border=0></td></tr>
-						<tr height=1><td bgcolor="#ffffff"><img height=1 src=$_REQUEST{__uri}0.gif width=1 border=0></td></tr>
-					</table></td>
+						<table width=90% border=0 cellspacing=0 cellpadding=0 align=center minheight=2>
+							<tr height=1><td bgcolor="#888888"><img height=1 src=$_REQUEST{__uri}0.gif width=1 border=0></td></tr>
+							<tr height=1><td bgcolor="#ffffff"><img height=1 src=$_REQUEST{__uri}0.gif width=1 border=0></td></tr>
+						</table>
+					</td>
 					<td width=1 bgcolor=#888888><img height=2 src=$_REQUEST{__uri}0.gif width=1 border=0></td>
 					<td width=1 bgcolor=#424142><img height=2 src=$_REQUEST{__uri}0.gif width=1 border=0></td>
 					
@@ -1342,11 +1374,22 @@ EOH
 		}
 		else {
 		
+			my $td = $type -> {items} ? <<EOH : qq{<td nowrap onmouseover="m_on(this)" onmouseout="m_off(this)" onclick="$$type{onclick}" class="vert-menu">&nbsp;&nbsp;$$type{label}&nbsp;&nbsp;</td>};
+				<td onmouseover="blockEvent ();" onmouseout="blockEvent ();">
+				<table width=100% border=0 cellspacing=0 cellpadding=0>
+					<tr>
+						<td         nowrap onmouseover="m_on(this)" onmouseout="m_off(this)" onclick="$$type{onclick}" class="vert-menu">&nbsp;&nbsp;$$type{label}&nbsp;&nbsp;</td>
+						<td width=5 nowrap onmouseover="m_on(this)" onmouseout="m_off(this)" onclick="$$type{onclick}" class="vert-menu" align=right><font face=Marlett>8</font>&nbsp;</td>
+					</tr>
+				</table>
+				</td>
+EOH
+		
 			$html .= <<EOH;
 				<tr>
 					<td width=1 bgcolor=#D6D3CE><img height=1 src=$_REQUEST{__uri}0.gif width=1 border=0></td>
 					<td width=1 bgcolor=#ffffff><img height=1 src=$_REQUEST{__uri}0.gif width=1 border=0></td>
-					<td nowrap onmouseover="m_on(this)" onmouseout="m_off(this)" onclick="$$type{onclick}" class="vert-menu">&nbsp;&nbsp;$$type{label}&nbsp;&nbsp;</td>
+					$td
 					<td width=1 bgcolor=#888888><img height=1 src=$_REQUEST{__uri}0.gif width=1 border=0></td>
 					<td width=1 bgcolor=#424142><img height=1 src=$_REQUEST{__uri}0.gif width=1 border=0></td>
 				</tr>
@@ -1368,11 +1411,14 @@ EOH
 		</div>
 EOH
 
+	foreach my $type (@$types) {
+		$html .= $type -> {vert_menu};
+	}
+
 	return $html;
 
 }
 
-
 ################################################################################
 # TABLES
 ################################################################################
@@ -1393,13 +1439,6 @@ sub draw_text_cell {
 
 	my ($_SKIN, $data, $options) = @_;
 	
-	unless ($data -> {attributes} -> {style}) {
-		$data -> {attributes} -> {class} ||= 
-			$options -> {is_total} ? 'row-cell-total' : 
-			$data -> {attributes} -> {bgcolor} ? 'row-cell-transparent' : 
-			'row-cell';
-	}	
-
 	my $attributes = dump_attributes ($data -> {attributes});
 
 	my $txt = '';
@@ -1410,14 +1449,6 @@ sub draw_text_cell {
 		$txt =~ s{^\s+}{};
 		$txt =~ s{\s+$}{};
 
-		unless ($_REQUEST {xls}) {
-			$txt = '&nbsp;' . $txt . '&nbsp;';
-		}
-
-		unless ($data -> {no_nobr}) {
-			$txt = '<nobr>' . $txt . '</nobr>';
-		}
-
 		if ($data -> {bold} || $options -> {bold}) {
 			$txt = '<b>' . $txt . '</b>';
 		}
@@ -1430,15 +1461,649 @@ sub draw_text_cell {
 			$txt = '<strike>' . $txt . '</strike>';
 		}
 		
-		if ($data -> {href}) {
-			$data -> {target} ||= '_self';
-			$txt = qq { <a class=$$data{a_class} target="$$data{target}" href="$$data{href}" onFocus="blur()">$txt</a> };
+		$txt = '&nbsp;' . $txt . '&nbsp;';
+
+		unless ($data -> {no_nobr}) {
+			$txt = '<nobr>' . $txt . '</nobr>';
 		}
 
+		if ($data -> {href}) {
+			$data -> {target} ||= '_self';
+			$txt = qq { <a id="$$data{a_id}" class=$$data{a_class} target="$$data{target}" href="$$data{href}" onFocus="blur()">$txt</a> };
+		}
+
+		if ($data -> {status}) {
+			$txt = qq {<img src='status_$data->{status}->{icon}.gif' border=0 alt='$data->{status}->{label}' align=absmiddle hspace=5>} . $txt;
+		}		
+
 	}
-			
+
+
 	return qq {\n\t<td $attributes>$txt</td>};
 
 }
 
+################################################################################
+
+sub draw_radio_cell {
+
+	my ($_SKIN, $data, $options) = @_;
+
+	my $attributes = dump_attributes ($data -> {attributes});
+
+	return qq {<td $$options{data} $attributes><input type=radio name=$$data{name} $$data{checked} value='$$data{value}'></td>};
+
+}
+
+################################################################################
+
+sub draw_checkbox_cell {
+
+	my ($_SKIN, $data, $options) = @_;
+
+	my $attributes = dump_attributes ($data -> {attributes});
+
+	return qq {<td $$options{data} $attributes><input type=checkbox name=$$data{name} $$data{checked} value='$$data{value}'></td>};
+
+}
+
+################################################################################
+
+sub draw_select_cell {
+
+	my ($_SKIN, $data, $options) = @_;
+
+	my $attributes = dump_attributes ($data -> {attributes});
+
+	my $multiple = $data -> {rows} > 1 ? "multiple size=$$data{rows}" : '';
+	my $html = qq {<td $attributes><nobr><select name="$$data{name}" onChange="is_dirty=true; $$options{onChange}" onkeypress="typeAhead()" $multiple>};
+
+	$html .= qq {<option value="0">$$data{empty}</option>\n} if defined $data -> {empty};
+
+	foreach my $value (@{$data -> {values}}) {
+		$html .= qq {<option value="$$value{id}" $$value{selected}>$$value{label}</option>\n};
+	}
+	
+	$html .= qq {</select></nobr></td>};
+	
+	return $html;
+
+}
+
+################################################################################
+
+sub draw_input_cell {
+
+	my ($_SKIN, $data, $options) = @_;
+
+	my $attributes = dump_attributes ($data -> {attributes});
+
+	return qq {<td $$data{title} $attributes><nobr><input onFocus="q_is_focused = true; left_right_blocked = true;" onBlur="q_is_focused = false; left_right_blocked = false;" type="text" name="$$data{name}" value="$$data{label}" maxlength="$$data{max_len}" size="$$data{size}"></nobr></td>};
+
+}
+
+################################################################################
+
+sub draw_row_button {
+
+	my ($_SKIN, $options) = @_;
+
+	if ($options -> {off} || $_REQUEST {lpt}) {	
+		return $conf -> {core_hide_row_buttons} == 2 ? '' : '<td class=bgr0 valign=top nowrap width="1%">&nbsp;</td>';
+	}
+	
+	if ($conf -> {core_show_icons} || $_REQUEST {__core_show_icons}) {
+		my $label = $options -> {label};
+		$options -> {label} = qq|<img src="/i/buttons/$$options{icon}.gif" alt="$$options{label}" border=0 hspace=0 vspace=0 align=absmiddle>|;
+		$options -> {label} .= "&nbsp;$label" if $options -> {force_label} || $conf -> {core_hide_row_buttons} > -1;
+	}
+	else {
+		$options -> {label} = "\&nbsp;[$$options{label}]\&nbsp;";
+	}
+	
+	my $vert_line = {label => $options -> {label}, href => $options -> {href}};
+	$vert_line -> {label} =~ s{[\[\]]}{}g;
+	push @{$_SKIN -> {__current_row} -> {__types}}, $vert_line;
+		
+	if ($conf -> {core_hide_row_buttons} == 2) {
+		return '';
+	}
+	elsif ($conf -> {core_hide_row_buttons} == 1) {
+		return $_SKIN -> draw_text_cell ({label => '&nbsp;'});
+	}
+	else {
+		return qq {<td $$options{title} class="row-button" valign=top nowrap width="1%"><a TABINDEX=-1 class="row-button" href="$$options{href}" target="$$options{target}">$$options{label}</a></td>};
+	}
+
+}
+
+####################################################################
+
+sub draw_table_header {
+	
+	my ($_SKIN, $data_rows, $html_rows) = @_;
+	my $html = '<thead>';
+	foreach (@$html_rows) {$html .= $_};
+	$html .= '</thead>';
+	
+}
+
+####################################################################
+
+sub draw_table_header_row {
+	
+	my ($_SKIN, $data_cells, $html_cells) = @_;
+	
+	my $html = '<tr>';
+	foreach (@$html_cells) {$html .= $_};
+	$html .= '</tr>';
+	
+	return $html;
+	
+}
+
+####################################################################
+
+sub draw_table_header_cell {
+	
+	my ($_SKIN, $cell) = @_;
+	
+	return '' if $cell -> {hidden} || $cell -> {off} || (!$cell -> {label} && $conf -> {core_hide_row_buttons} == 2);
+
+	$cell -> {label} .= "\&nbsp;\&nbsp;<a class=lnk4 href=\"$$cell{href_asc}\"><b>\&uarr;</b></a>"  if $cell -> {href_asc};
+	$cell -> {label} .= "\&nbsp;\&nbsp;<a class=lnk4 href=\"$$cell{href_desc}\"><b>\&darr;</b></a>" if $cell -> {href_desc};
+
+	if ($cell -> {href}) {
+		$cell -> {label} = "<a class=lnk4 href=\"$$cell{href}\"><b>" . $cell -> {label} . "</b></a>";
+	}	
+
+	my $attributes = dump_attributes ($cell -> {attributes});
+	
+	return "<th $attributes $$cell{title}>\&nbsp;$$cell{label}\&nbsp;</th>";
+
+}
+
+####################################################################
+
+sub draw_table {
+
+	my ($_SKIN, $tr_callback, $list, $options) = @_;
+
+	$options -> {height}     ||= 10000;
+	$options -> {min_height} ||= 200;
+	
+	$$options{toolbar} =~ s{^\s+}{}sm;
+	$$options{toolbar} =~ s{\s+$}{}sm;
+
+	my $html = <<EOH;
+	
+		$$options{title}
+		$$options{path}
+		$$options{top_toolbar}
+		
+		<table cellspacing=0 cellpadding=0 width="100%">		
+			<tr>		
+				<form name=$$options{name} action=$_REQUEST{__uri} method=post enctype=multipart/form-data target=invisible>
+					<input type=hidden name=type value=$$options{type}>
+					<input type=hidden name=action value=$$options{action}>
+					<input type=hidden name=sid value=$_REQUEST{sid}>
+					<input type=hidden name=__last_query_string value="$_REQUEST{__last_last_query_string}">
+					<input type=hidden name=__last_scrollable_table_row value="$_REQUEST{__last_scrollable_table_row}">
+EOH
+
+	foreach my $key (keys %_REQUEST) {
+		next if $key =~ /^_/ or $key =~/^(type|action|sid|__last_query_string)$/;
+		$html .= qq {<input type=hidden name=$key value="$_REQUEST{$key}">\n};
+	}
+
+	$html .= qq {<td class=bgr8><div class="table-container" style="height: expression(actual_table_height(this,$$options{min_height},$$options{height},'$__last_centered_toolbar_id'));"><table cellspacing=1 cellpadding=0 width="100%" id="scrollable_table" lpt=$$options{lpt}>\n};
+
+	$html .= $options -> {header} if $options -> {header};
+
+	$html .= qq {<tbody>\n};
+
+	if ($options -> {dotdot}) {
+		$html .= $options -> {dotdot};
+	}
+
+	my $menus = '';
+
+	foreach our $i (@$list) {
+		
+		foreach my $tr (@{$i -> {__trs}}) {
+			
+			
+			$html .= "<tr id='$$i{__tr_id}'";
+			
+			if (@{$i -> {__types}} && $conf -> {core_hide_row_buttons} > -1 && !$_REQUEST {lpt}) {
+				$menus .= $i -> {__menu};
+				$html  .= qq{ oncontextmenu="open_popup_menu('$i'); blockEvent ();"};
+			}
+
+			$html .= '>';
+			$html .= $tr;
+			$html .= '</tr>';
+			
+		}
+		
+	}
+
+	$html .= <<EOH;
+			</tbody></table></div>$$options{toolbar}</td></form></tr></table>
+		$menus
+		
+EOH
+
+	$__last_centered_toolbar_id = '';
+		
+	return $html;
+
+}
+
+################################################################################
+
+sub draw_one_cell_table {
+
+	my ($_SKIN, $options, $body) = @_;
+	
+	return <<EOH			
+		<table cellspacing=0 cellpadding=0 width="100%">
+				<form name=form action=$_REQUEST{__uri} method=post enctype=multipart/form-data target=invisible>
+					<tr><td class=bgr8>$body</td></tr>
+				</form>
+		</table>
+EOH
+
+}
+
+################################################################################
+
+sub draw_error_page {
+
+	my ($_SKIN, $page) = @_;
+
+	my $html = <<EOH;
+		<html>
+			<head></head>
+			<body onLoad="
+EOH
+
+	if ($page -> {error_field}) {
+		$html .= <<EOJ;
+			var e = window.parent.document.getElementsByName('$page->{error_field}'); 
+			if (e && e[0]) { e[0].focus () }				
+EOJ
+	}
+
+	my $message = js_escape ($_REQUEST {error});
+								
+	$html .= <<EOH;
+		history.go (-1); 
+		alert ($message);
+		window.parent.document.body.style.cursor = 'normal';
+	">
+				</body>
+			</html>				
+EOH
+
+	return $html;
+
+}
+
+################################################################################
+
+sub start_page {
+}
+
+################################################################################
+
+sub draw_page {
+
+	my ($_SKIN, $page) = @_;
+
+	if ($_REQUEST {__only_form}) {
+
+		$page -> {body} =~ s{\\}{\\\\}gsm;
+		$page -> {body} =~ s{\"}{\\\"}gsm; #"
+		$page -> {body} =~ s{[\n\r\s]+}{ }gsm;
+
+		return <<EOH;
+		<body onLoad="main()">
+			<script>
+				function main () {
+					window.parent.document.forms ['$_REQUEST{__only_form}'].elements ['_$_REQUEST{__only_field}'].outerHTML = "$page->{body}";
+					window.parent.document.forms ['$_REQUEST{__only_form}'].elements ['_$_REQUEST{__only_field}'].focus ();
+				}
+			</script>
+		</body>
+EOH
+	}
+		
+	$_REQUEST {__scrollable_table_row} ||= 0;
+	
+	my $meta_refresh = $_REQUEST {__meta_refresh} ? qq{<META HTTP-EQUIV=Refresh CONTENT="$_REQUEST{__meta_refresh}; URL=@{[create_url()]}&__no_focus=1">} : '';	
+
+	my $root = $_REQUEST{__uri};
+	
+	my $request_package = ref $apr;
+	my $mod_perl = $ENV {MOD_PERL};
+	$mod_perl ||= 'NO mod_perl AT ALL';
+	
+	my $timeout = 1000 * (60 * $conf -> {session_timeout} - 1);
+	
+	$_REQUEST {__select_rows} += 0;
+	
+	my $static_salt = $Zanas_VERSION_NAME;
+	$static_salt .= '_00000';
+	if ($_REQUEST {sid}) {
+		$static_salt .= $_REQUEST {sid};
+	}
+	else {
+		$static_salt .= $$ . time ();
+		$static_salt =~ s{\.}{}g;
+	}
+						
+	return <<EOH;
+		<html>		
+			<head>
+				<title>$$i18n{_page_title}</title>
+								
+				<meta name="Generator" content="Zanas ${Zanas::VERSION} / $$SQL_VERSION{string}; parameters are fetched with $request_package; gateway_interface is $ENV{GATEWAY_INTERFACE}; $mod_perl is in use">
+				<meta http-equiv=Content-Type content="text/html; charset=$$i18n{_charset}">
+				
+				$meta_refresh
+				
+				<LINK href="${root}zanas_${static_salt}.css" type=text/css rel=STYLESHEET>
+				@{[ map {<<EOJS} @{$_REQUEST{__include_css}} ]}
+					<LINK href="/i/$_.css" type=text/css rel=STYLESHEET>
+EOJS
+
+					<script src="${root}navigation_${static_salt}.js">
+					</script>
+				@{[ map {<<EOCSS} @{$_REQUEST{__include_js}} ]}
+					<script type="text/javascript" src="/i/${_}.js">
+					</script>
+EOCSS
+			
+				<script>
+					var select_rows = $_REQUEST{__select_rows};
+					var scrollable_table = null;
+					var scrollable_table_row = 0;
+					var scrollable_table_row_id = 0;
+					var scrollable_table_row = 0;
+					var scrollable_table_row_length = 0;
+					var scrollable_table_row_cell_old_style = '';
+					var is_dirty = false;					
+					var scrollable_table_is_blocked = false;
+					var q_is_focused = false;					
+					var left_right_blocked = false;					
+					var scrollable_rows = new Array();		
+					var td2sr = new Array ();
+					var td2sc = new Array ();
+					var last_vert_menu = new Array ();
+					last_vert_menu [0] = null;
+					last_vert_menu [1] = null;
+					last_vert_menu [2] = null;
+					last_vert_menu [3] = null;
+					last_vert_menu [4] = null;
+					var last_main_menu = null;
+					var ms_word = null;
+					
+					function td_on_click () {
+						var uid = window.event.srcElement.uniqueID;
+						var new_scrollable_table_row = td2sr [uid];
+						var new_scrollable_table_row_cell = td2sc [uid];
+						if (new_scrollable_table_row == null || new_scrollable_table_row_cell == null) return;
+						scrollable_rows [scrollable_table_row].cells [scrollable_table_row_cell].className = scrollable_table_row_cell_old_style;
+						scrollable_table_row = new_scrollable_table_row;
+						scrollable_table_row_cell = new_scrollable_table_row_cell;
+						scrollable_table_row_cell_old_style = scrollable_rows [scrollable_table_row].cells [scrollable_table_row_cell].className;
+						scrollable_rows [scrollable_table_row].cells [scrollable_table_row_cell].className += ' row-cell-hilite';
+						focus_on_first_input (scrollable_rows [scrollable_table_row].cells [scrollable_table_row_cell]);
+						return false;
+					}
+					
+					
+					function body_on_load () {
+
+						@{[ $_REQUEST{__no_focus} ? '' : 'window.focus ();' ]}
+
+						@{[ $_REQUEST{sid} ? <<EOK : '' ]}
+							keepaliveID = setTimeout ("open('$_REQUEST{__uri}?keepalive=$_REQUEST{sid}', 'invisible'); clearTimeout (keepaliveID)", $timeout);
+EOK
+
+						$_REQUEST{__doc_on_load}
+
+						@{[ $_REQUEST {__pack} ? <<EOF : '']}
+							var newWidth  = document.all ['bodyArea'].offsetWidth + 10;
+							var newHeight = document.all ['bodyArea'].offsetHeight + 30;
+							window.resizeTo (newWidth, newHeight);						
+							window.moveTo ((screen.width - newWidth) / 2, (screen.height - newHeight) / 2);
+EOF
+
+						if (!document.body.getElementsByTagName) return;
+
+						var tables = document.body.getElementsByTagName ('table');
+
+						if (tables != null) {										
+							for (var i = 0; i < tables.length; i++) {
+
+								if (tables [i].id != 'scrollable_table') continue;
+
+								var rows = tables [i].tBodies (0).rows;
+
+								for (var j = 0; j < rows.length; j++) {
+									scrollable_rows = scrollable_rows.concat (rows [j]);
+								}
+							}					
+						}
+
+						for (var i = 0; i < scrollable_rows.length; i++) {
+
+							var cells = scrollable_rows [i].cells;
+							for (var j = 0; j < cells.length; j++) {
+								var scrollable_cell = cells [j];
+								td2sr [scrollable_cell.uniqueID] = i;
+								td2sc [scrollable_cell.uniqueID] = j;
+								scrollable_cell.onclick = td_on_click;
+								scrollable_cell.oncontextmenu = td_on_click;
+							}
+						}
+
+						scrollable_table = document.getElementById ('scrollable_table');
+
+						if (scrollable_table) {				
+
+							scrollable_table = scrollable_table.tBodies (0);
+
+							scrollable_table_row = $_REQUEST{__scrollable_table_row};
+							scrollable_table_row_cell = 0;
+
+							if (scrollable_rows.length > 0) {
+								if (scrollable_table_row > scrollable_rows.length - 1) {
+									scrollable_table_row = scrollable_rows.length - 1;
+								}
+								scrollable_table_row_cell_old_style = scrollable_rows [scrollable_table_row].cells [scrollable_table_row_cell].className;
+								scrollable_rows [scrollable_table_row].cells [scrollable_table_row_cell].className += ' row-cell-hilite';
+							}
+							else {
+								scrollable_table = null;
+							}
+
+						}
+
+						var focused_inputs = document.getElementsByName ('$_REQUEST{__focused_input}');
+
+						if (focused_inputs != null && focused_inputs.length > 0) {
+							var focused_input = focused_inputs [0];
+							focused_input.focus ();
+							if (focused_input.type == 'radio') {
+								focused_input.select ();
+							}
+						}
+						else {	
+
+							var forms = document.forms;
+							if (forms != null) {
+
+								var done = 0;
+
+								for (var i = 0; i < forms.length; i++) {
+
+									var elements = forms [i].elements;
+
+									if (elements != null) {
+
+										for (var j = 0; j < elements.length; j++) {
+
+											var element = elements [j];
+
+											if (element.tagName == 'INPUT' && element.name == 'q') {
+												break;
+											}
+
+											if (
+												   (element.tagName == 'INPUT'  && (element.type == 'text' || element.type == 'checkbox' || element.type == 'radio'))
+												||  element.tagName == 'TEXTAREA') 
+											{
+												element.focus ();
+												done = 1;
+												break;
+											}										
+
+										}									
+
+									}
+
+									if (done) {
+										break;
+									}
+
+								}
+
+							}
+
+						}
+
+						@{[ $_REQUEST {__blur_all} ? <<EOF : '']}
+
+						if (inputs != null) {										
+							for (var i = 0; i < inputs.length; i++) {
+								inputs [i].blur ();
+							}					
+						}
+
+EOF
+
+						$_REQUEST{__on_load}
+					
+					}
+					
+					
+				</script>
+				
+			</head>
+			<body 
+				bgcolor=white 
+				leftMargin=0 
+				topMargin=0 
+				marginwidth=0 
+				marginheight=0 
+				name="body" 
+				id="body"
+				scroll="auto"
+				onload= "body_on_load (); try {StartClock ()} catch (e) {}"
+				onbeforeunload="document.body.style.cursor = 'wait'"
+				onunload=" try {KillClock ()} catch (e) {}"
+				onmousemove="check_popup_menus(event)"
+				onkeydown="
+				
+					if (window.event.keyCode == 88 && window.event.altKey) {
+						document.location.href = '$_REQUEST{__uri}?type=_logout&sid=$_REQUEST{sid}&salt=@{[rand]}';
+						blockEvent ();
+					}
+					
+					handle_basic_navigation_keys ();
+					
+					@{[ map {&{"handle_hotkey_$$_{type}"} ($_)} @{$page->{scan2names}} ]}
+					
+				"						
+			>
+				
+				@{[ $_REQUEST{__help_url} ? <<EOHELP : '' ]}
+					<script for="body" event="onhelp">
+						nope ('$_REQUEST{__help_url}', '_blank', 'toolbar=no,resizable=yes');
+						event.returnValue = false;
+					</script>						
+EOHELP
+				
+				<div id="bodyArea" _style='height:100%; padding:0px; margin:0px'>
+					<iframe name=invisible src="${root}0.html" width=0 height=0 application="yes" style="display:none">
+					</iframe>
+					$$page{auth_toolbar}
+					$$page{menu}
+					$$page{body}
+				</div>				
+			</body>
+		</html>
+EOH
+
+}
+
+################################################################################
+
+sub handle_hotkey_focus {
+
+	my ($r) = @_;
+	
+	<<EOJS
+		if (window.event.keyCode == $$r{code} && window.event.altKey && window.event.ctrlKey) {
+			document.form.$$r{data}.focus ();
+			blockEvent ();
+		}
+EOJS
+
+}
+
+################################################################################
+
+sub handle_hotkey_href {
+
+	my ($r) = @_;
+	
+	my $ctrl = $r -> {ctrl} ? '' : '!';
+	my $alt  = $r -> {alt}  ? '' : '!';
+	
+	my $condition = 
+		$r -> {off}     ? '0' :
+		$r -> {confirm} ? 'window.confirm(' . js_escape ($r -> {confirm}) . ')' : 
+		'1';
+
+	if ($r -> {href}) {
+			
+		return <<EOJS
+			if (window.event.keyCode == $$r{code} && $alt window.event.altKey && $ctrl window.event.ctrlKey) {
+				if ($condition) {
+					nope ('$$r{href}&__from_table=1&salt=' + Math.random () + '&' + scrollable_rows [scrollable_table_row].id, '_self');
+				}
+				blockEvent ();
+			}
+EOJS
+
+	}
+	else {
+		
+		return <<EOJS
+			if (window.event.keyCode == $$r{code} && $alt window.event.altKey && $ctrl window.event.ctrlKey) {
+				if ($condition) {
+					var a = document.getElementById ('$$r{data}');
+					activate_link (a.href, a.target);
+				}
+				blockEvent ();
+			}
+EOJS
+	}
+
+}
+
 1;
\ No newline at end of file
