@@ -98,26 +98,34 @@ function activate_link (href, target) {
 
 }
 
-function open_popup_menu (type) {
+function open_popup_menu (type, level) {
 
 	var div = document.getElementById ('vert_menu_' + type);
 	
 	if (!div) return;
 	
-	if (last_vert_menu) {last_vert_menu.style.display = 'none'; last_vert_menu = null}
+	if (!level) level = 0;
+	
+	for (var i = level; i < last_vert_menu.length; i++) {
+		if (last_vert_menu [i]) last_vert_menu [i].style.display = 'none';
+		last_vert_menu [i] = null;
+	}
 
 	var mainMenuCell = document.getElementById ('main_menu_' + type);
 	
 	if (mainMenuCell) {
 		div.style.top  = mainMenuCell.offsetTop  + mainMenuCell.offsetParent.offsetTop  + mainMenuCell.offsetParent.offsetParent.offsetTop  + 16;
 		div.style.left = mainMenuCell.offsetLeft + mainMenuCell.offsetParent.offsetLeft + mainMenuCell.offsetParent.offsetParent.offsetLeft - 6;
-		last_vert_menu = div;
 	}
 	else {
 		div.style.top  = event.y - 5 + document.body.scrollTop;
 		div.style.left = event.x - 5 + document.body.scrollLeft;
 	}
 	
+	if (mainMenuCell || level > 0) {
+		last_vert_menu [level] = div;
+	}
+	
 	div.style.display = 'block';
 	
 }
@@ -419,17 +427,43 @@ function msword_line (s) {
 }
 
 function m_on (td) {
-	td.style.background='#08246b';
-	td.style.color='white';
+	var cells = td.parentElement.cells;
+	for (var i = 0; i < cells.length; i++) {
+		if (cells [i].className != 'vert-menu') continue;
+		cells [i].style.background='#08246b';
+		cells [i].style.color='white';
+	}
 	blockEvent ();
 }
 
 function m_off (td) {
-	td.style.background='#D6D3CE';
-	td.style.color='black';
+	var cells = td.parentElement.cells;
+	for (var i = 0; i < cells.length; i++) {
+		if (cells [i].className != 'vert-menu') continue;
+		cells [i].style.background='#D6D3CE';
+		cells [i].style.color='black';
+	}
 	blockEvent ();
 }
 
+function check_popup_menus (event) {
+	
+	var activeLevel = -1;
+	
+	for (var i = last_vert_menu.length; i >= 0; i--) {
+		if (!last_vert_menu [i]) continue;
+		if (!hasMouse (last_vert_menu [i], event)) continue;
+		activeLevel = i;
+		break;
+	}
+			
+	for (var i = activeLevel + 1; i < last_vert_menu.length; i++) {					
+		if (last_vert_menu [i]) last_vert_menu [i].style.display = 'none';
+		last_vert_menu [i] = null;
+	}
+
+}
+
 function actual_table_height (table, min_height, height, id_toolbar) {
 
 	var real_height       = table.firstChild.offsetHeight;
