@@ -42,12 +42,45 @@ warn "relink $table_name: $old_id -> $new_id";
 warn "relink $$column_def{table_name} ($$column_def{name}): $old_id -> $new_id";
 
 			if ($column_def -> {TYPE_NAME} =~ /int/) {
+			
+				sql_do (<<EOS, $old_id);
+					INSERT INTO __moved_links
+						(table_name, column_name, id_from, id_to)
+					SELECT
+						'$$column_def{table_name}' AS table_name,
+						'$$column_def{name}' AS column_name,
+						id AS id_from,
+						$old_id AS id_to
+					FROM
+						$$column_def{table_name}
+					WHERE
+						$$column_def{name} = ?
+EOS
+
 				sql_do ("UPDATE $$column_def{table_name} SET $$column_def{name} = ? WHERE $$column_def{name} = ?", $new_id, $old_id);
+				
 			}
 			else {
+			
 				$old_id = ',' . $old_id . ',';
 				$new_id = ',' . $new_id . ',';
+			
+				sql_do (<<EOS, '%' . $old_id . '%');
+					INSERT INTO __moved_links
+						(table_name, column_name, id_from, id_to)
+					SELECT
+						'$$column_def{table_name}' AS table_name,
+						'$$column_def{name}' AS column_name,
+						id AS id_from,
+						$old_id AS id_to
+					FROM
+						$$column_def{table_name}
+					WHERE
+						$$column_def{name} LIKE ?
+EOS
+
 				sql_do ("UPDATE $$column_def{table_name} SET $$column_def{name} = REPLACE($$column_def{name}, ?, ?) WHERE $$column_def{name} LIKE ?", $old_id, $new_id, '%' . $old_id . '%');
+
 			}
 
 		}
@@ -64,6 +97,53 @@ warn "relink $$column_def{table_name} ($$column_def{name}): $old_id -> $new_id";
 
 ################################################################################
 
+sub sql_undo_relink {
+
+	my ($table_name, $old_ids) = @_;
+	
+	ref $old_ids eq ARRAY or $old_ids = [$old_ids];
+			
+warn Dumper ($DB_MODEL -> {tables} -> {$table_name});
+warn Dumper ($DB_MODEL -> {tables} -> {$table_name} -> {references});
+
+	foreach my $old_id (@$old_ids) {
+	
+warn "undo relink $table_name: $old_id";
+
+		my $record = sql_select_hash ($table_name, $old_id);
+		
+		foreach my $column_def (@{$DB_MODEL -> {tables} -> {$table_name} -> {references}}) {
+
+			my $from = <<EOS;
+				FROM
+					__moved_links
+				WHERE
+					table_name = '$$column_def{table_name}'
+					AND column_name = '$$column_def{name}'
+					AND id_to = $old_id
+EOS
+
+			my $ids = sql_select_ids ("SELECT id_from $from");
+			sql_do ("DELETE $from");
+
+warn "undo relink $$column_def{table_name} ($$column_def{name}): $old_id";
+
+			if ($column_def -> {TYPE_NAME} =~ /int/) {
+				sql_do ("UPDATE $$column_def{table_name} SET $$column_def{name} = ? WHERE id IN ($ids)", $old_id);
+			}
+			else {			
+				$old_id = $old_id . ',';
+				sql_do ("UPDATE $$column_def{table_name} SET $$column_def{name} = CONCAT($$column_def{name}, ?) WHERE id IN ($ids)", $old_id);
+			}
+
+		}
+		
+	}
+	
+}
+
+################################################################################
+
 sub sql_version {
 
 	my $version = {	string => 'MySQL ' . sql_select_scalar ('SELECT VERSION()') };
@@ -207,8 +287,9 @@ sub sql_select_all_cnt {
 
 	if ($_REQUEST {xls} && $conf -> {core_unlimit_xls} && !$_REQUEST {__limit_xls}) {
 		$sql =~ s{LIMIT.*}{}ism;
-		my $result = sql_select_all ($sql, @params);
-		return ($result, 0 + @$result);
+		my $result = sql_select_all ($sql, @params, $options);
+		my $cnt = ref $result eq ARRAY ? 0 + @$result : -1;
+		return ($result, $cnt);
 	}
 
 	if ((!$conf -> {core_infty} && $_REQUEST {__infty}) || ($conf -> {core_infty} && !$_REQUEST {__no_infty})) {
@@ -217,10 +298,11 @@ sub sql_select_all_cnt {
 		
 		my ($start, $portion) = ($1, $2);
 				
-		my $result = sql_select_all ($sql, @params);
+		my $result = sql_select_all ($sql, @params, $options);
+		my $cnt = ref $result eq ARRAY ? 0 + @$result : 0;
 				
 		if (0 + @$result <= $portion) {
-			return ($result, $start + @$result);
+			return ($result, $start + $cnt);
 		}
 		else {
 			pop @$result;
@@ -236,6 +318,9 @@ sub sql_select_all_cnt {
 	
 	my $st = $db -> prepare ($sql);
 	$st -> execute (@params);
+	
+	return $st if $options -> {no_buffering};
+	
 	my $result = $st -> fetchall_arrayref ({});	
 	$st -> finish;
 
@@ -294,6 +379,9 @@ sub sql_select_all {
 	
 	my $st = $db -> prepare ($sql);
 	$st -> execute (@params);
+
+	return $st if $options -> {no_buffering};
+
 	my $result = $st -> fetchall_arrayref ({});	
 	$st -> finish;
 	
